"""
This type stub file was generated by pyright.
"""

from pathlib import Path
from typing import Generator

"""Build definition check utilities."""
IGNORE_MISSING = ...

def unroll_subdirs(target: Path, rel_path: Path) -> str:
    """Opens a meson.build file and returns the file with literal subdir loops converted
    to single static assignment form.
    """

def inspect_files(target: Path, rel_path: Path, found_files: list[str], extra_files: list[str]) -> dict[str, list[str]]:
    ...

def process(target: Path, rel_path: Path, found_files: list[str] | None = ...) -> dict[str, list[str]]:
    """Process an OZI project build definition's files."""

def validate(target: Path, rel_path: Path, subdirs: list[str], children: set[str] | None) -> Generator[Path, None, None]:
    """Validate an OZI standard build definition's directories."""

def walk(
    target: Path,
    rel_path: Path,
    found_files: list[str] | None = None,
    project_name: str | None = None,
) -> Generator[dict[Path, dict[str, list[str]]], None, None]:
    """Walk an OZI standard build definition directory."""
