"""
This type stub file was generated by pyright.
"""

from __future__ import annotations

import sys
from argparse import Namespace

"""ozi-new: quick-start OZI project creation script."""
if sys.platform != 'win32':
    ...  # import curses
else:
    ...

def project(project: Namespace) -> None:
    """Create a new project in a target directory."""

def wrap(project: Namespace) -> None:
    """Create a new wrap file for publishing. Not a public function."""

def main(args: list[str] | None = ...) -> None:
    """Main ozi.new entrypoint."""

