"""
This type stub file was generated by pyright.
"""

from __future__ import annotations

from typing import TYPE_CHECKING
from typing import NoReturn

"""ozi-fix: Project fix script that outputs a meson rewriter JSON array."""
if TYPE_CHECKING:
    ...
def main() -> NoReturn:
    """Main ozi.fix entrypoint."""

