"""
This type stub file was generated by pyright.
"""

"""Core OZI packaging management plane module.

.. versionremoved:: 1.2
   The module ``filters`` was moved to ``blastpipe.ozi_templates.filters``

.. versionchanged:: 1.12

   The ozi_templates module was moved out of blastpipe.
"""

def current_version() -> str:
    """Returns the currently installed version of OZI.

    .. versionchanged:: 1.14

      Previously was part of the ozi.spec submodule.

    """

__version__: str
__author__: str
__all__ = ('__version__', '__author__', '__doc__')
